#include <conio.h>
#include <stdio.h>
#include <dos.h>
#include <my\mytypes.h>

typedef struct RMREGS
{
        DWORD           edi;
        DWORD           esi;
        DWORD           ebp;
        DWORD           reserved;
        DWORD           ebx;
        DWORD           edx;
        DWORD           ecx;
        DWORD           eax;
        UWORD           flags;
        UWORD           es,ds,fs,gs,ip,cs,sp,ss;
} RMREGS;

typedef struct VesaInfo
{
        char            Signature[4];
        UWORD           Version;
        UWORD           OEMNameOffset;
        UWORD           OEMNameSegment;                 //Pointer to OEM name?
        UBYTE           Capabilities[4];
        UWORD           SupportedModesOffset;
        UWORD           SupportedModesSegment;          //Pointer to list of supported VESA and OEM modes (terminated with 0xffff).
        UBYTE           Reserved[238];
} VesaInfo;

typedef struct VesaModeData
{
        UWORD           ModeAttributes;
        UBYTE           WindowAAttributes;
        UBYTE           WindowBAttributes;
        UWORD           WindowGranularity;
        UWORD           WindowSize;
        UWORD           StartSegmentWindowA;
        UWORD           StartSegmentWindowB;
        void            (*WindowPositioningFunction)(DWORD page);
        UWORD           BytesPerScanLine;

        //Remainder of this structure is optional for VESA modes in v1.0/1.1, needed for OEM modes.

        UWORD           PixelWidth;
        UWORD           PixelHeight;
        UBYTE           CharacterCellPixelWidth;
        UBYTE           CharacterCellPixelHeight;
        UBYTE           NumberOfMemoryPlanes;
        UBYTE           BitsPerPixel;
        UBYTE           NumberOfBanks;
        UBYTE           MemoryModelType;
        UBYTE           SizeOfBank;
        UBYTE           NumberOfImagePages;
        UBYTE           Reserved1;

        //VBE v1.2+

        UBYTE           RedMaskSize;
        UBYTE           RedFieldPosition;
        UBYTE           GreenMaskSize;
        UBYTE           GreenFieldPosition;
        UBYTE           BlueMaskSize;
        UBYTE           BlueFieldPosition;
        UBYTE           ReservedMaskSize;
        UBYTE           ReservedFieldPosition;
        UBYTE           DirectColourModeInfo;

    //VBE v2.0+

    DWORD       PhysicalAddress;
    UBYTE       Reserved2[212];
} VesaModeData;


void GetVesaInfo(VesaInfo far *block)
{
 union REGS regs;
 struct SREGS sregs;
 VesaInfo far *dosblock;
 UWORD segment,selector;
 RMREGS   rmregs;

 memset(&rmregs,0,sizeof(rmregs));
 memset(&regs,0,sizeof(regs));
 memset(&sregs,0,sizeof(sregs));

 //dos memoria foglalas:
 regs.w.ax=0x100;
 regs.w.bx=(UWORD)(sizeof(VesaInfo)+15)/16;   //Paragraph-ra kerekitunk
 int386(0x31,&regs,&regs);
 segment=regs.w.ax;
 selector=regs.w.dx;
 dosblock=MK_FP(selector,0);


 //Vesa lekerd
 segread(&sregs);
 rmregs.eax = 0x4f00;
 rmregs.es = segment;
 rmregs.ds = segment;
 memset(&regs,0,sizeof(regs));
 regs.x.eax = 0x300;
 regs.x.ebx = 0x10;
 regs.x.ecx = 0;
 sregs.es = FP_SEG(&rmregs);
 regs.x.edi = FP_OFF(&rmregs);
 int386x(0x31,&regs,&regs,&sregs);       //Get vesa info.

 //atmenteni magunknak
 _fmemcpy(block,dosblock,sizeof(VesaInfo));

 //Dos memoria vissza
 memset(&regs,0,sizeof(regs));
 regs.w.ax=0x101;
 regs.w.dx=selector;
 int386(0x31,&regs,&regs);
}


void GetModeInfo(VesaModeData far *block,int mode)
{
 union REGS regs;
 struct SREGS sregs;
 VesaInfo far *dosblock;
 UWORD segment,selector;
 RMREGS   rmregs;

 memset(&rmregs,0,sizeof(rmregs));
 memset(&regs,0,sizeof(regs));
 memset(&sregs,0,sizeof(sregs));

 //dos memoria foglalas:
 regs.w.ax=0x100;
 regs.w.bx=(UWORD)(sizeof(VesaModeData)+15)/16;   //Paragraph-ra kerekitunk
 int386(0x31,&regs,&regs);
 segment=regs.w.ax;
 selector=regs.w.dx;
 dosblock=MK_FP(selector,0);


 //Vesa lekerd
 segread(&sregs);
 rmregs.eax = 0x4f01;
 rmregs.ecx = mode;
 rmregs.es = segment;
 rmregs.ds = segment;
 memset(&regs,0,sizeof(regs));
 regs.x.eax = 0x300;
 regs.x.ebx = 0x10;
 sregs.es = FP_SEG(&rmregs);
 regs.x.edi = FP_OFF(&rmregs);
 int386x(0x31,&regs,&regs,&sregs);       //Get vesa info.

 //atmenteni magunknak
 _fmemcpy(block,dosblock,sizeof(VesaModeData));

 //Dos memoria vissza
 memset(&regs,0,sizeof(regs));
 regs.w.ax=0x101;
 regs.w.dx=selector;
 int386(0x31,&regs,&regs);
}



void main(void)
{
 VesaInfo VesaInfo;
 VesaModeData VesaModeData;
 char *realaddress;

 GetVesaInfo(&VesaInfo);
 printf("%c%c%c%c  ",VesaInfo.Signature[0],VesaInfo.Signature[1],VesaInfo.Signature[2],VesaInfo.Signature[3]);
 printf("%d.%d \n",VesaInfo.Version>>8,VesaInfo.Version%256);
 realaddress=VesaInfo.OEMNameOffset+16*VesaInfo.OEMNameSegment;    //0 a baziscim
 printf("%s\n",realaddress);
 GetModeInfo(&VesaModeData,0x10f);
 printf("Linear Frame Buffer at: %xH",VesaModeData.PhysicalAddress);
}